/** INCLUDES *******************************************************/
#include "mscext.h"
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "FreqMeter.h"

#include "Freq_DAC.h"

/** V A R I A B L E S ********************************************************/
#pragma udata

volatile DWORD_VAL MeasuredFreq;
volatile WORD_VAL UpperFreq;
volatile BYTE InterruptCount;

/** D E C L A R A T I O N S **************************************************/
#pragma code

void InitFreqMeter(void)
{
	// set up timer1: off, internal, /8 prescaler
	PIR1bits.TMR1IF = 0;
	PIE1bits.TMR1IE = 1;
	IPR1bits.TMR1IP = 1;
	T1CON = 0xb0;//0b10110000;

	// set up timer0: off, 16bit, external, no prescaler
	INTCONbits.TMR0IF = 0;
	INTCONbits.TMR0IE = 1;
	TRISAbits.TRISA4 = 1;
	T0CON = 0x38;//0b00111000;

	MeasuredFreq.Val = 0x00;
}

void StartFreqCount(void)
{
	TMR1H = 0x00;
	TMR1L = 0x00;
	TMR0H = 0x00;
	TMR0L = 0x00;
	
	UpperFreq.Val = 0;
	InterruptCount = 1;

	INTCONbits.TMR0IF = 0;
	T0CONbits.TMR0ON = 1;
	T1CONbits.TMR1ON = 1;
}

void StopFreqCount(void)
{
	T1CONbits.TMR1ON = 0;
	T0CONbits.TMR0ON = 0;
}

void FreqMeter_OnTMR1IF(void)
{
	if (InterruptCount == 0)
	{
		StopFreqCount();

		if (INTCONbits.TMR0IF == 1)
			UpperFreq.Val ++;

		MeasuredFreq.byte.LB = TMR0L;
		MeasuredFreq.byte.HB = TMR0H;
		MeasuredFreq.byte.UB = UpperFreq.byte.LB;
		MeasuredFreq.byte.MB = UpperFreq.byte.HB;

		FreqDAC_AutoHoldFreq(MeasuredFreq);

		StartFreqCount();
	}
	else
	{
		InterruptCount--;
	}
}

void FreqMeter_OnTMR0IF(void)
{
	UpperFreq.Val ++;
}